//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * 四次元テクスチャ座標ヘッダ
 * @author Junpee
 */

#ifndef TEX_COORD_4_H_
#define TEX_COORD_4_H_

#include <Core/System/Math.h>
#include <Core/Primitive/String.h>

namespace Lamp{

//------------------------------------------------------------------------------
/**
 * 四次元テクスチャ座標
 *
 * このクラスは継承しないで下さい。
 */
class TexCoord4{
public:
	//--------------------------------------------------------------------------
	// メンバ変数
	//--------------------------------------------------------------------------
	/// メンバ変数
	union{
		/// 各要素
		struct{
			/// U値
			float u;
			/// V値
			float v;
			/// W値
			float w;
			/// X値
			float x;
		};

		/// 配列
		float array[4];
	};

	//--------------------------------------------------------------------------
	// 定数
	//--------------------------------------------------------------------------
	/// ゼロ座標
	static const TexCoord4 zero;

	/// 単位座標
	static const TexCoord4 unit;

	/// U単位座標
	static const TexCoord4 unitU;

	/// V単位座標
	static const TexCoord4 unitV;

	/// W単位座標
	static const TexCoord4 unitW;

	/// X単位座標
	static const TexCoord4 unitX;

	//--------------------------------------------------------------------------
	// コンストラクタ
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 *
	 * このコンストラクタは初期値の設定を行わないため値は不定です。
	 */
	inline TexCoord4(){}

	/**
	 * コンストラクタ
	 * @param sourceU Uの初期値
	 * @param sourceV Vの初期値
	 * @param sourceW Wの初期値
	 * @param sourceX Xの初期値
	 */
	inline TexCoord4(
		float sourceU, float sourceV, float sourceW, float sourceX) :
		u(sourceU), v(sourceV), w(sourceW), x(sourceX){
	}

	/**
	 * コンストラクタ
	 * @param source 初期値配列
	 */
	inline explicit TexCoord4(const float* const source) :
		u(source[0]), v(source[1]), w(source[2]), x(source[3]){
	}

	//--------------------------------------------------------------------------
	// 値の設定
	//--------------------------------------------------------------------------
	/**
	 * 値の設定
	 * @param sourceU Uの設定値
	 * @param sourceV Vの設定値
	 * @param sourceW Wの設定値
	 * @param sourceX Xの設定値
	 */
	inline void set(float sourceU, float sourceV, float sourceW, float sourceX){
		u = sourceU;
		v = sourceV;
		w = sourceW;
		x = sourceX;
	}

	/**
	 * 値の設定
	 * @param source 設定値配列
	 */
	inline void set(const float* const source){
		u = source[0];
		v = source[1];
		w = source[2];
		x = source[3];
	}

	//--------------------------------------------------------------------------
	// 演算
	//--------------------------------------------------------------------------
	/**
	 * 加算
	 * @param addCoord 加算する四次元テクスチャ座標
	 * @return 加算された四次元テクスチャ座標
	 */
	inline TexCoord4 operator +(const TexCoord4& addCoord) const{
		return TexCoord4(u + addCoord.u, v + addCoord.v,
			w + addCoord.w, x + addCoord.x);
	}

	/**
	 * 減算
	 * @param subCoord 減算する四次元テクスチャ座標
	 * @return 減算された四次元テクスチャ座標
	 */
	inline TexCoord4 operator -(const TexCoord4& subCoord) const{
		return TexCoord4(u - subCoord.u, v - subCoord.v,
			w - subCoord.w, x - subCoord.x);
	}

	/**
	 * 乗算
	 * @param mulValue 乗算する値
	 * @return 乗算された四次元テクスチャ座標
	 */
	inline TexCoord4 operator *(float mulValue) const{
		return TexCoord4(u * mulValue, v * mulValue,
			w * mulValue, x * mulValue);
	}

	/**
	 * 乗算
	 * @param mulValue 乗算する値
	 * @param mulCoord 乗算する四次元テクスチャ座標
	 * @return 乗算された四次元テクスチャ座標
	 */
	inline friend TexCoord4 operator *(
		float mulValue, const TexCoord4& mulCoord){
		return TexCoord4(mulCoord.u * mulValue, mulCoord.v * mulValue,
			mulCoord.w * mulValue, mulCoord.x * mulValue);
	}

	/**
	 * +演算子
	 * @return 四次元テクスチャ座標のコピー
	 */
	inline TexCoord4 operator +() const{ return *this; }

	/**
	 * -演算子
	 * @return 値の符号が反転した四次元テクスチャ座標
	 */
	inline TexCoord4 operator -() const{ return TexCoord4(-u, -v, -w, -x); }

	//--------------------------------------------------------------------------
	// 代入演算
	//--------------------------------------------------------------------------
	/**
	 * 代入加算
	 * @param addCoord 加算する四次元テクスチャ座標
	 * @return 加算された四次元テクスチャ座標
	 */
	inline TexCoord4& operator +=(const TexCoord4& addCoord){
		u += addCoord.u;
		v += addCoord.v;
		w += addCoord.w;
		x += addCoord.x;
		return *this;
	}

	/**
	 * 代入減算
	 * @param subCoord 減算する四次元テクスチャ座標
	 * @return 減算された四次元テクスチャ座標
	 */
	inline TexCoord4& operator -=(const TexCoord4& subCoord){
		u -= subCoord.u;
		v -= subCoord.v;
		w -= subCoord.w;
		x -= subCoord.x;
		return *this;
	}

	/**
	 * 代入乗算
	 * @param mulValue 乗算する値
	 * @return 乗算された四次元テクスチャ座標
	 */
	inline TexCoord4& operator *=(float mulValue){
		u *= mulValue;
		v *= mulValue;
		w *= mulValue;
		x *= mulValue;
		return *this;
	}

	//--------------------------------------------------------------------------
	// 論理演算
	//--------------------------------------------------------------------------
	/**
	 * 四次元テクスチャ座標が同じかどうか
	 * @param target 比較する四次元テクスチャ座標
	 * @return 同じ値であればtrueを返す
	 */
	inline bool operator ==(const TexCoord4& target) const{
		return ((u == target.u) && (v == target.v) &&
			(w == target.w) && (x == target.x));
	}

	/**
	 * 四次元テクスチャ座標が同じかどうか
	 * @param target 比較する四次元テクスチャ座標
	 * @param epsilon 誤差
	 * @return 誤差の範囲内で同じ値であればtrueを返す
	 */
	inline bool epsilonEquals(
		const TexCoord4& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (
			(Math::abs(u - target.u) <= epsilon) &&
			(Math::abs(v - target.v) <= epsilon) &&
			(Math::abs(w - target.w) <= epsilon) &&
			(Math::abs(x - target.x) <= epsilon));
	}

	/**
	 * 四次元テクスチャ座標が同じでないかどうか
	 * @param target 比較する四次元テクスチャ座標
	 * @return 同じでない値であればtrueを返す
	 */
	inline bool operator !=(const TexCoord4& target) const{
		return ((u != target.u) || (v != target.v) ||
			(w != target.w) || (x != target.x));
	}

	/**
	 * 四次元テクスチャ座標が同じでないかどうか
	 * @param target 比較する四次元テクスチャ座標
	 * @param epsilon 誤差
	 * @return 誤差の範囲内で同じでない値であればtrueを返す
	 */
	inline bool notEpsilonEquals(
		const TexCoord4& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (
			(Math::abs(u - target.u) > epsilon) ||
			(Math::abs(v - target.v) > epsilon) ||
			(Math::abs(w - target.w) > epsilon) ||
			(Math::abs(x - target.x) > epsilon));
	}

	//--------------------------------------------------------------------------
	// その他
	//--------------------------------------------------------------------------
	/**
	 * 文字列化
	 * @return 四次元テクスチャ座標の文字列表記
	 */
	inline String toString() const{
		String returnString;
		returnString.format("( %.8f, %.8f, %.8f, %.8f )", u, v, w, x);
		return returnString;
	}

	//--------------------------------------------------------------------------
private:

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of TEX_COORD_4_H_
//------------------------------------------------------------------------------
